@@ -56,7 +56,7 @@ type toget =
    crypt : toget_crypt}
 
 let toget ?(crypt = NoChange) input position length =
-  {input; position; length; crypt}
+  {input = input; position = position; length = length; crypt = crypt}
 
 let length_of_toget t = t.length
 let position_of_toget t = t.position
@@ -228,6 +228,7 @@ let rec remove_dict_entry dict key =
 for streams. *)
 let rec replace_dict_entry dict key value =
   match dict with
+  | Null -> Dictionary (replace key value [])
   | Dictionary d -> Dictionary (replace key value d)
   | Stream ({contents = (dict', stream)} as s) ->
       s := (replace_dict_entry dict' key value, stream);
@@ -237,6 +238,7 @@ let rec replace_dict_entry dict key value =
 (* Add a dict entry, replacing if there. Also works for streams. *)
 let rec add_dict_entry dict key value =
   match dict with
+  | Null -> Dictionary (add key value [])
   | Dictionary d -> Dictionary (add key value d)
   | Stream ({contents = (dict', stream)} as s) ->
       s := (add_dict_entry dict' key value, stream);
@@ -245,7 +247,7 @@ let rec add_dict_entry dict key value =
 
 (* Get a stream from disk if it hasn't already been got. *)
 let getstream = function
-  | Stream ({contents = (d, ToGet {input = i; position = o; length = l; crypt})} as stream) ->
+  | Stream ({contents = (d, ToGet {input = i; position = o; length = l; crypt = crypt})} as stream) ->
       if l = 0 then stream := (d, Got (mkbytes 0)) else
         begin try
           let data =
@@ -439,7 +441,7 @@ let lookup_exception (exp : exn) pdf key dict =
   let dict' =
     match direct pdf dict with
     | Dictionary d | Stream {contents = Dictionary d, _} -> d
-    | _ -> raise (PDFError "not a dictionary")
+    | _ -> raise (PDFError ("lookup_exception: not a dictionary, key = " ^ key))
   in
     match lookup key dict' with
     | None -> raise exp
