@@ -1,6 +1,11 @@
 (* General Input and Output *)
 open Pdfutil
 
+(* We used to have a type called "bytes" before OCaml did.  In order
+   not to break client code using e.g. Pdfio.bytes, we keep the name
+   but expose an alias caml_bytes for the built-in type. *)
+type caml_bytes = bytes
+
 (* External type for big streams of bytes passed to C*)
 type rawbytes =
   (int, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t
@@ -8,7 +13,7 @@ type rawbytes =
 (* But for speed, we use strings of length < Sys.max_string_length *)
 type bytes =
   | Long of rawbytes
-  | Short of string
+  | Short of caml_bytes
 
 let bigarray_unsafe_get =
   Bigarray.Array1.unsafe_get
@@ -19,9 +24,9 @@ let bigarray_unsafe_set =
 (* Extract the raw bytes, without necessarily copying *)
 let raw_of_bytes = function
   | Short b ->
-      let l = Bigarray.Array1.create Bigarray.int8_unsigned Bigarray.c_layout (String.length b) in
-        for x = 0 to String.length b - 1 do
-          bigarray_unsafe_set l x (int_of_char (String.unsafe_get b x))
+      let l = Bigarray.Array1.create Bigarray.int8_unsigned Bigarray.c_layout (Bytes.length b) in
+        for x = 0 to Bytes.length b - 1 do
+          bigarray_unsafe_set l x (int_of_char (Bytes.unsafe_get b x))
         done;
         l
   | Long b -> b
@@ -32,32 +37,32 @@ let bytes_of_raw b = Long b
 (* Make a stream of a given size. *)
 let mkbytes l =
   if l <= Sys.max_string_length
-    then Short (String.create l)
+    then Short (Bytes.create l)
     else Long (Bigarray.Array1.create Bigarray.int8_unsigned Bigarray.c_layout l)
 
 (* Find the size of a stream. *)
 let bytes_size = function
-  | Short s -> String.length s
+  | Short s -> Bytes.length s
   | Long b -> Bigarray.Array1.dim b
 
 let bset s n v =
   match s with
-  | Short s -> s.[n] <- Char.unsafe_chr v
+  | Short s -> Bytes.set s n (Char.unsafe_chr v)
   | Long s -> Bigarray.Array1.set s n v
 
 let bset_unsafe s n v =
   match s with
-  | Short s -> String.unsafe_set s n (Char.unsafe_chr v)
+  | Short s -> Bytes.unsafe_set s n (Char.unsafe_chr v)
   | Long s -> bigarray_unsafe_set s n v
 
 let bget s n =
   match s with
-  | Short s -> int_of_char (s.[n])
+  | Short s -> int_of_char (Bytes.get s n)
   | Long s -> Bigarray.Array1.get s n
 
 let bget_unsafe s n =
   match s with
-  | Short s -> int_of_char (String.unsafe_get s n)
+  | Short s -> int_of_char (Bytes.unsafe_get s n)
   | Long s -> bigarray_unsafe_get s n
 
 (* For lexing / parsing byte streams, keep the position. Starts at zero. *)
@@ -85,6 +90,15 @@ let bytes_of_string s =
         done;
       stream
 
+let bytes_of_caml_bytes s =
+  let l = Bytes.length s in
+    let stream = mkbytes l in
+      if l > 0 then
+        for k = 0 to l - 1 do
+          bset_unsafe stream k (int_of_char (Bytes.unsafe_get s k))
+        done;
+      stream
+
 (* Make a byte stream from an integer list. *)
 let bytes_of_list l =
   let length = length l in
@@ -218,7 +232,7 @@ let input_of_channel ?(source = "channel") ch =
      in_channel_length =
        in_channel_length ch;
      set_offset =
-       (fun o -> offset := o);
+       (fun o -> if !offset = 0 then offset := o);
      caml_channel = Some ch;
      source = source}
 
@@ -246,7 +260,7 @@ let input_of_stream ?(source = "bytes") s =
          in_channel_length =
            ssize;
          set_offset =
-           (fun o -> offset := o);
+           (fun o -> if !offset = 0 then offset := o);
          caml_channel = None;
          source = source}
 
@@ -443,30 +457,30 @@ let setinit i s o l =
       | Short s ->
           begin match i.caml_channel with
           | None ->
-              for x = o to o + l - 1 do String.unsafe_set s x (Char.unsafe_chr (i.input_byte ())) done
+              for x = o to o + l - 1 do Bytes.unsafe_set s x (Char.unsafe_chr (i.input_byte ())) done
           | Some ch -> really_input ch s o l
           end
       | Long s ->
           for x = o to o + l - 1 do bigarray_unsafe_set s x (i.input_byte ()) done
 
 let setinit_string i s o l =
-  let max = String.length s - 1
+  let max = Bytes.length s - 1
   and last = o + 1 - 1 in
     if o > max || o < 0 || last < 0 || last > max then raise (Failure "setinit_string") else
       match i.caml_channel with
       | Some ch ->
           really_input ch s o l
       | None ->
-          for x = o to o + l - 1 do String.unsafe_set s x (Char.unsafe_chr (i.input_byte ())) done
+          for x = o to o + l - 1 do Bytes.unsafe_set s x (Char.unsafe_chr (i.input_byte ())) done
 
 let bytes_of_input i o l =
   i.seek_in o;
-  let s = String.create l in
+  let s = Bytes.create l in
     setinit_string i s 0 l;
     if l <= Sys.max_string_length then
       Short s
     else
-      bytes_of_string s
+      bytes_of_caml_bytes s
 
 let getinit i s o l =
   let max = bytes_size s - 1
@@ -475,7 +489,7 @@ let getinit i s o l =
       match s with
       | Short s ->
           begin match i.out_caml_channel with
-          | None -> for x = o to o + l - 1 do i.output_byte (int_of_char (String.unsafe_get s x)) done
+          | None -> for x = o to o + l - 1 do i.output_byte (int_of_char (Bytes.unsafe_get s x)) done
           | Some ch -> output ch s o l
           end
       | Long s ->
@@ -653,13 +667,11 @@ let write_bitstream_append_aligned a b =
 let debug_next_char i =
   try
     let a = unopt (i.input_char ()) in
-      Printf.printf "%C = %i\n" a (int_of_char a)
+      Printf.eprintf "%C = %i\n" a (int_of_char a)
   with
     _ -> ()
 
 let debug_next_n_chars n i =
   for x = 1 to n do debug_next_char i done;
-  flprint "\n";
+  prerr_string "\n";
   for x = 1 to n do rewind i done
-
-
