@@ -54,6 +54,9 @@ val add_root : int -> (string * Pdf.pdfobject) list -> Pdf.t -> Pdf.t
 (** Number of pages in a document, faster than reading the pages and counting. *)
 val endpage : Pdf.t -> int
 
+(** Find a page indirect from the page tree of a document, given a page number. *)
+val page_object_number : Pdf.t -> int -> int option
+
 (** {2 Compound operations} *)
 
 (** Rename the resources within a number of page resource dictionaries and
@@ -78,6 +81,8 @@ val shortest_unused_prefix : Pdf.t -> string
 
 val add_prefix : Pdf.t -> string -> unit
 
+val protect : Pdf.t -> Pdf.pdfobject -> Pdf.pdfobject list -> Pdfops.t list
+
 val prepend_operators : Pdf.t -> Pdfops.t list -> ?fast:bool -> t -> t
 
 val postpend_operators : Pdf.t -> Pdfops.t list -> ?fast:bool -> t -> t
@@ -87,7 +92,7 @@ words they are all explicit. This is required before writing a file with
 linearization *)
 (*val pagetree_make_explicit : Pdf.t -> Pdf.t*)
 
-val pagenumber_of_target : ?refnums:int list -> Pdf.t -> Pdfdest.t -> int
+val pagenumber_of_target : ?fastrefnums:(int, int) Hashtbl.t -> Pdf.t -> Pdfdest.t -> int
 
 val target_of_pagenumber : Pdf.t -> int -> Pdfdest.t
 
